/*
 * Laboratory for Scientific Computing
 * http://www.lam-mpi.org/tutorials/
 * University of Notre Dame
 *
 * MPI Tutorial
 * Lab 2: The cannonical ring program
 *
 * Mail questions regarding tutorial material to mpi@lam-mpi.org
 */

#include <iostream.h>
#include "mpi++.h"


int main(int argc, char *argv[])
{
  int num; 
  int rank, size, tag, next, from;
  MPI::Status status;

  /* Start up MPI */

  MPI::Init(argc, argv);
  rank = MPI::COMM_WORLD.Get_rank();
  size = MPI::COMM_WORLD.Get_size();
 
  /* Arbitrarily choose 201 to be our tag.  Calculate the */
  /* rank of the next process in the ring.  Use the modulus */
  /* operator so that the last process "wraps around" to rank */
  /* zero. */

  tag = 201;
  next = (rank+1) % size;
  from = MPI::ANY_SOURCE;

  /* If we are the "console" process, get a integer from the */
  /* user to specify how many times we want to go around the */
  /* ring */

  if (rank == 0) {
    cout << "Enter the number of times around the ring: ";
    cin >> num;

    cout << "Process " << rank << " sending " << num << " to " << next << endl;
    /* Send the number to the next process in the ring */

  }

  /* Pass the message around the ring.  The exit mechanism works */
  /* as follows: the message (a positive integer) is passed */
  /* around the ring.  Each time is passes rank 0, it is decremented. */
  /* When each processes receives the 0 message, it passes it on */
  /* to the next process and then quits.  By passing the 0 first, */
  /* every process gets the 0 message and can quit normally. */

  while(1) {

    /* Receive the number */

    cout << "Process " << rank << " received " << num << endl;

    if (rank == 0) {
      num--;
      cout << "Process 0 decremented num" << endl;
    }

    cout << "Process " << rank << " sending " << num << " to " << next << endl;

    /* Send the number on to the next process */


    if (num == 0) {
      cout << "Process " << rank << " exiting" << endl;
      break;
    }
  }

  /* The last process does one extra send to process 0, which needs */
  /* to be received before the program can exit */

  if (rank == 0) {
    /* Receive the number */

  }

  /* Quit */

  MPI::Finalize();
  return 0;
}
